import { ipcRenderer, remote, IpcRendererEvent } from 'electron';

import { Observable, Subject } from 'rxjs';

import { NativeService, WindowState } from '@ucap-webmessenger/native';
import { Channel } from '../types/channel.type';
import { share } from 'rxjs/operators';

export class ElectronNativeService implements NativeService {
  private windowStateChangedSubject: Subject<WindowState> | null = null;
  private windowStateChanged$: Observable<WindowState> | null = null;

  showNotify(
    roomSeq: number,
    title: string,
    contents: string,
    image: string,
    useSound: boolean
  ): void {
    ipcRenderer.send(
      Channel.showNotify,
      roomSeq,
      title,
      contents,
      image,
      useSound
    );
  }

  checkForUpdates(): Observable<boolean> {
    return new Observable<boolean>(subscriber => {
      try {
        subscriber.next(ipcRenderer.sendSync(Channel.checkForUpdates));
      } catch (error) {
        subscriber.error(error);
      } finally {
        subscriber.complete();
      }
    });
  }

  showImageViewer(): void {
    ipcRenderer.send(Channel.showImageViewer);
  }

  readFile(path: string): Observable<ArrayBuffer> {
    return new Observable<ArrayBuffer>(subscriber => {
      try {
        subscriber.next(ipcRenderer.sendSync(Channel.readFile, path));
      } catch (error) {
        subscriber.error(error);
      } finally {
        subscriber.complete();
      }
    });
  }

  saveFile(path: string, buf: ArrayBuffer): Observable<boolean> {
    return new Observable<boolean>(subscriber => {
      try {
        subscriber.next(ipcRenderer.sendSync(Channel.saveFile, path, buf));
      } catch (error) {
        subscriber.error(error);
      } finally {
        subscriber.complete();
      }
    });
  }

  windowStateChanged(): Observable<WindowState> {
    if (!this.windowStateChangedSubject) {
      this.windowStateChangedSubject = new Subject<WindowState>();
      this.windowStateChanged$ = this.windowStateChangedSubject
        .asObservable()
        .pipe(share());
    }

    ipcRenderer.on(
      Channel.windowStateChanged,
      (event: IpcRendererEvent, windowState: WindowState) => {
        console.log('windowStateChanged', windowState);
        this.windowStateChangedSubject.next(windowState);
      }
    );
    return this.windowStateChanged$;
  }

  windowClose(): void {
    const currentWindow = remote.getCurrentWindow();
    if (!currentWindow) {
      return;
    }

    currentWindow.close();
  }

  windowMinimize(): void {
    const currentWindow = remote.getCurrentWindow();
    if (!currentWindow) {
      return;
    }

    currentWindow.minimize();
  }

  windowMaximize(): void {
    const currentWindow = remote.getCurrentWindow();
    if (!currentWindow) {
      return;
    }

    if (currentWindow.isMaximized()) {
      currentWindow.unmaximize();
    } else {
      currentWindow.maximize();
    }
  }

  constructor() {}
}
